


#if defined(_WP8) || defined(_WINRT_ARM)
#define LIGHT_COUNT 1
#else
#define LIGHT_COUNT 8
#endif

Texture2D u_MaterialDiffuseTexture;
SamplerState u_MaterialDiffuseTextureSampler;

cbuffer cbGlobal
{
	matrix u_WorldMatrix;
	matrix u_WorldViewProjectionMatrix;
	matrix u_WorldMatrixInverseTranspose;
	matrix u_TextureTransformMatrix;


	float4 u_CameraPosition;
	float4 u_MaterialAmbientColor;
	float4 u_MaterialDiffuseColor;
	float4 u_MaterialSpecularColor;

	float u_MaterialShininess;
	float u_LightingEnabled;
	float u_VertexColorEnabled;
	float u_MaterialHasDiffuseTexture;



	float4	u_GlobalAmbientColor;
	float	u_ActiveLights[LIGHT_COUNT];
	float4	u_LightPositions[LIGHT_COUNT];
	float4	u_LightDiffuseColors[LIGHT_COUNT];
	float4	u_LightSpecularColors[LIGHT_COUNT];

};

struct VS_IN
{
	float3 a_Position : POSITION;
	float3 a_Normal : NORMAL;
	float2 a_TexCoord : TEXCOORD;
	float4 a_Color : COLOR;
	uint4 a_BlendIndices : BLENDINDICES;
	float4 a_BlendWeight : BLENDWEIGHT;
};
#ifdef _WINRT
#include "Spark2Projects/_GeeaEngine/Media/Include/Animation_dx11.inc"
#else
#include "Include/Animation_dx11.inc"

#endif

struct PS_IN
{
	float4 v_Position : SV_POSITION;
	float2 v_TexCoord : TEXCOORD;
	float4 v_FrontColor : COLOR;
};

float4 GetLightingColor(float4 ambientColor, float4 diffuseColor, float4 specularColor, float shininess, float3 P, float3 N, float3 C)
{
	float4 color = mul(ambientColor , u_GlobalAmbientColor );

	for (int i = 0; i < LIGHT_COUNT; ++i)
	{
		if (u_ActiveLights[i] == 1.0)
		{
			float3 L = normalize(u_LightPositions[i].xyz - mul( P , u_LightPositions[i].w ) );
			float3 H = normalize(L + C);
			
			float diff_factor = max(0.0, dot(L, N));
			float spec_factor = max(0.0, dot(H, N));
			
			color += mul(mul(diffuseColor , u_LightDiffuseColors[i]) , diff_factor );
			color += mul(mul(specularColor , u_LightSpecularColors[i]) , pow(spec_factor, shininess) );
		}
	}
	
	return clamp( color, float4(0.0, 0.0, 0.0, 0.0), float4(1.0, 1.0, 1.0, 1.0) );
}




PS_IN vs_main(VS_IN In)
{
	
	PS_IN Out;
	

	float4x4 animationMatrix	= GetAnimationMatrix(In);
	
	if (u_LightingEnabled == 1.0)
	{
		float4 matAmbientColor;
		float4 matDiffuseColor;
		float4 matSpecularColor;
		float matShininess;

		if (u_VertexColorEnabled == 0.0)
		{
			matAmbientColor = u_MaterialAmbientColor;
			matDiffuseColor = u_MaterialDiffuseColor;
			matSpecularColor = u_MaterialSpecularColor;
			matShininess = u_MaterialShininess;
		}
		else
		{
			matAmbientColor = In.a_Color/255.0;
			matDiffuseColor = In.a_Color/255.0;
			matSpecularColor = float4(0.0, 0.0, 0.0, 0.0);
			matShininess = 0.0;
		}

		//float3 N = normalize( matrix (u_WorldMatrixInverseTranspose) * transpose(inverse(matrix (animationMatrix))) * In.a_Normal);
		float3 N = normalize( mul ( mul ( (float3x3) u_WorldMatrixInverseTranspose , (float3x3)animationMatrix ) , In.a_Normal ) );
		float3 P = ( mul ( u_WorldMatrix ,float4(In.a_Position, 1.0 ) ) ).xyz;
		float3 C = normalize(u_CameraPosition.xyz - P);

		Out.v_FrontColor = GetLightingColor(matAmbientColor, matDiffuseColor, matSpecularColor, matShininess, P, N, C);
	}
	else
	{
		if (u_VertexColorEnabled == 0.0)
		{
			Out.v_FrontColor = u_MaterialAmbientColor;
		}
		else
		{
			Out.v_FrontColor = In.a_Color/255.0;
		}
	}
	Out.v_TexCoord = ( mul ( u_TextureTransformMatrix , float4(In.a_TexCoord, 0.0, 1.0) ) ).xy;
	Out.v_Position = mul ( animationMatrix ,float4(In.a_Position,1.0) ) ;
	Out.v_Position.w = 1.0 ;
	Out.v_Position = mul(u_WorldViewProjectionMatrix , Out.v_Position );

	return Out;
};

float4 ps_main(PS_IN In) : SV_TARGET
{
    float4 result ;
	result = lerp(In.v_FrontColor, u_MaterialDiffuseTexture.Sample(u_MaterialDiffuseTextureSampler, In.v_TexCoord) * In.v_FrontColor, u_MaterialHasDiffuseTexture);
    return result;
}