//------------------------------------------------------
// uniforms
//------------------------------------------------------
Texture2D u_MaterialDiffuseTexture;
SamplerState u_MaterialDiffuseTextureSampler;
cbuffer cbGlobal
{
 matrix 	u_WorldViewProjectionMatrix;
 float4 	u_InnerColor;
 float4  	u_OutlineColor;
};

//------------------------------------------------------
// vertex attributes
//------------------------------------------------------
struct VS_IN
{
float3	a_Position  :   POSITION;
float2	a_TexCoord  :   TEXCOORD;
float4	a_Color  	:  	COLOR;
};

//------------------------------------------------------
// varying variables
//------------------------------------------------------
struct PS_IN
{
float4 v_position 		: SV_POSITION;
float2	v_TexCoord 		: TEXCOORD;
float4	v_FrontColor    : COLOR;
};

PS_IN vs_main(VS_IN In)
{
    PS_IN Out;
	Out.v_TexCoord = In.a_TexCoord;
	Out.v_FrontColor = In.a_Color;
	Out.v_position =  mul( u_WorldViewProjectionMatrix, float4(In.a_Position, 1.0));
    return Out;
}


float4 ps_main(PS_IN In)  : SV_TARGET
{

	float2 ret = (u_MaterialDiffuseTexture.Sample(u_MaterialDiffuseTextureSampler, In.v_TexCoord)).ar;
	float innerFill = ret.r;
	float outerFill = ret.g;

	return float4(u_OutlineColor.r + (u_InnerColor.r - u_OutlineColor.r) * innerFill,
		u_OutlineColor.g + (u_InnerColor.g - u_OutlineColor.g) * innerFill,
		u_OutlineColor.b + (u_InnerColor.b - u_OutlineColor.b) * innerFill,
		min(innerFill   * u_InnerColor.a + (outerFill - innerFill) * u_OutlineColor.a, 1.0));
}