//=================================================================================================
// Constant buffers
//=================================================================================================

cbuffer cbGlobal
{
	matrix u_ProjectionMatrix;
	float u_FarClipDistance;
	float4 u_ViewLightPos;
};

cbuffer cbObject
{
	matrix u_WorldViewMatrix;
	matrix u_WorldViewMatrixInverseTranspose;
	matrix u_WorldViewProjectionMatrix;
};

//=================================================================================================
// Structures
//=================================================================================================

struct VS_IN
{
    float3 Position : POSITION;
    float3 Normal   : NORMAL;
};

struct PS_IN
{
    float4 Position : SV_POSITION;
};

//=================================================================================================
// Vertex shader
//=================================================================================================

PS_IN vs_main(VS_IN In)
{
	PS_IN Out;

	// Vertex position in view space
	float4 viewPos = mul(u_WorldViewMatrix, float4(In.Position, 1.0f));
	
	// Face normal in view space
	float4 viewNormal = mul(u_WorldViewMatrixInverseTranspose, float4(In.Normal, 0.0f));
	
	// Light direction in view space
	float4 viewLightDir = lerp(u_ViewLightPos, viewPos - u_ViewLightPos, u_ViewLightPos.w);
	
	// Extrude vertex if it is facing away from the light
	if (dot(viewNormal, -viewLightDir) < 0.0f)
	{
		if (viewLightDir.y > 0.0f)
			viewPos.xyz += viewLightDir.xyz * (0.99f * u_FarClipDistance - viewPos.y) / viewLightDir.y;
		else
			viewPos = viewLightDir;

		Out.Position = mul(u_ProjectionMatrix, viewPos);
	}
	else
	{
		Out.Position = mul(u_WorldViewProjectionMatrix, float4(In.Position, 1.0f));
	}
	
	return Out;
}

//=================================================================================================
// Pixel shader
//=================================================================================================

float4 ps_main(PS_IN In) : SV_TARGET
{
	return float4(1.0f, 1.0f, 1.0f, 1.0f);
}
