// Transformation matrices
float4x4 u_WorldViewMatrix;
float4x4 u_WorldViewMatrixInverseTranspose;
float4x4 u_ProjectionMatrix;
float4x4 u_WorldViewProjectionMatrix;

// Far clip distance
float u_FarClipDistance;

// Light position in view space
float4 u_ViewLightPos;

float4 vs_main(float4 a_Position : POSITION, float3 a_Normal : NORMAL) : POSITION
{
	// Vertex position in view space
	float4 viewPos = mul(a_Position, u_WorldViewMatrix);
	
	// Face normal in view space
	float4 viewNormal = mul(float4(a_Normal, 0), u_WorldViewMatrixInverseTranspose);
	
	// Light direction in view space
	float4 viewLightDir = lerp(u_ViewLightPos, viewPos - u_ViewLightPos, u_ViewLightPos.w);
	
	// Extrude vertex if it is facing away from the light
	if (dot(viewNormal, -viewLightDir) < 0)
	{
		if (viewLightDir.y > 0)
			viewPos.xyz += viewLightDir.xyz * (0.99 * u_FarClipDistance - viewPos.y) / viewLightDir.y;
		else
			viewPos = viewLightDir;

		return mul(viewPos, u_ProjectionMatrix);
	}
	else
	{
		return mul(a_Position, u_WorldViewProjectionMatrix);
	}
}

float4 ps_main() : COLOR
{
	return float4(1, 1, 1, 1);
}
